
LIBRARY "CoinMP"

EXPORTS
	CoinInitSolver
	CoinFreeSolver
	CoinGetSolverName
	CoinGetSolverNameBuf
	CoinGetVersionStr
	CoinGetVersionStrBuf
	CoinGetVersion
	CoinGetFeatures
	CoinGetMethods
	CoinGetInfinity
	CoinCreateProblem
	CoinLoadMatrix
	CoinLoadNames
	CoinLoadNamesBuf
	CoinLoadProblem
	CoinLoadProblemBuf
	CoinLoadInitValues
	CoinLoadInteger
	CoinLoadPriority
	CoinLoadSos
	CoinLoadSemiCont
	CoinLoadQuadratic
	CoinLoadNonlinear
	CoinUnloadProblem
	CoinCheckProblem
	CoinGetProblemName
	CoinGetProblemNameBuf
	CoinGetColCount
	CoinGetRowCount
	CoinGetColName
	CoinGetColNameBuf
	CoinGetRowName
	CoinGetRowNameBuf
	CoinRegisterMsgLogCallback
	CoinRegisterLPIterCallback
	CoinRegisterMipNodeCallback
	CoinSetMsgLogCallback
	CoinSetIterCallback
	CoinSetMipNodeCallback
	CoinOptimizeProblem
	CoinGetSolutionStatus
	CoinGetSolutionText
	CoinGetSolutionTextBuf
	CoinGetObjectValue
	CoinGetMipBestBound
	CoinGetIterCount
	CoinGetMipNodeCount
	CoinGetSolutionValues
	CoinGetSolutionRanges
	CoinGetSolutionBasis
	CoinReadFile
	CoinWriteFile
	CoinOpenLogFile
	CoinCloseLogFile
	CoinGetOptionCount
	CoinLocateOptionID
	CoinLocateOptionName
	CoinGetOptionID
	CoinGetOptionInfo
	CoinGetIntOptionMinMax
	CoinGetRealOptionMinMax
	CoinGetOptionNamesBuf
	CoinGetOptionGroup
	CoinGetOptionType
	CoinGetIntOptionDefaultValue
	CoinGetIntOptionMinValue
	CoinGetIntOptionMaxValue
	CoinGetRealOptionDefaultValue
	CoinGetRealOptionMinValue
	CoinGetRealOptionMaxValue
	CoinGetOptionName
	CoinGetOptionNameBuf
	CoinGetOptionShortName
	CoinGetOptionShortNameBuf
	CoinGetOptionChanged
	CoinGetIntOption
	CoinSetIntOption
	CoinGetRealOption
	CoinSetRealOption
	CoinGetStringOption
	CoinGetStringOptionBuf
	CoinSetStringOption
	Clp_newModel
	Clp_deleteModel
	ClpSolve_new
	ClpSolve_delete
	Clp_loadProblem
	Clp_loadQuadraticObjective
	Clp_readMps
	Clp_copyInIntegerInformation
	Clp_deleteIntegerInformation
	Clp_resize
	Clp_deleteRows
	Clp_addRows
	Clp_deleteColumns
	Clp_addColumns
	Clp_chgRowLower
	Clp_chgRowUpper
	Clp_chgColumnLower
	Clp_chgColumnUpper
	Clp_chgObjCoefficients
	Clp_dropNames
	Clp_copyNames
	Clp_numberRows
	Clp_numberColumns
	Clp_primalTolerance
	Clp_setPrimalTolerance
	Clp_dualTolerance
	Clp_setDualTolerance
	Clp_dualObjectiveLimit
	Clp_setDualObjectiveLimit
	Clp_objectiveOffset
	Clp_setObjectiveOffset
	Clp_problemName
	Clp_setProblemName
	Clp_numberIterations
	Clp_setNumberIterations
	Clp_setMaximumIterations
	Clp_maximumSeconds
	Clp_setMaximumSeconds
	Clp_hitMaximumIterations
	Clp_status
	Clp_setProblemStatus
	Clp_secondaryStatus
	Clp_setSecondaryStatus
	Clp_optimizationDirection
	Clp_setOptimizationDirection
	Clp_primalRowSolution
	Clp_primalColumnSolution
	Clp_dualRowSolution
	Clp_dualColumnSolution
	Clp_rowLower
	Clp_rowUpper
	Clp_objective
	Clp_columnLower
	Clp_columnUpper
	Clp_getNumElements
	Clp_getVectorStarts
	Clp_getIndices
	Clp_getVectorLengths
	Clp_getElements
	Clp_objectiveValue
	Clp_integerInformation
	Clp_infeasibilityRay
	Clp_unboundedRay
	Clp_statusExists
	Clp_statusArray
	Clp_copyinStatus
	Clp_getColumnStatus
	Clp_getRowStatus
	Clp_setColumnStatus
	Clp_setRowStatus
	Clp_setUserPointer
	Clp_getUserPointer
	Clp_registerCallBack
	Clp_clearCallBack
	Clp_setLogLevel
	Clp_logLevel
	Clp_lengthNames
	Clp_rowName
	Clp_columnName
	Clp_initialSolve
	Clp_initialSolveWithOptions
	Clp_initialDualSolve
	Clp_initialPrimalSolve
	Clp_initialBarrierSolve
	Clp_initialBarrierNoCrossSolve
	Clp_dual
	Clp_primal
	Clp_idiot
	Clp_scaling
	Clp_scalingFlag
	Clp_crash
	Clp_primalFeasible
	Clp_dualFeasible
	Clp_dualBound
	Clp_setDualBound
	Clp_infeasibilityCost
	Clp_setInfeasibilityCost
	Clp_perturbation
	Clp_setPerturbation
	Clp_algorithm
	Clp_setAlgorithm
	Clp_sumDualInfeasibilities
	Clp_numberDualInfeasibilities
	Clp_sumPrimalInfeasibilities
	Clp_numberPrimalInfeasibilities
	Clp_saveModel
	Clp_restoreModel
	Clp_checkSolution
	Clp_getNumRows
	Clp_getNumCols
	Clp_getIterationCount
	Clp_isAbandoned
	Clp_isProvenOptimal
	Clp_isProvenPrimalInfeasible
	Clp_isProvenDualInfeasible
	Clp_isPrimalObjectiveLimitReached
	Clp_isDualObjectiveLimitReached
	Clp_isIterationLimitReached
	Clp_getObjSense
	Clp_setObjSense
	Clp_getRowActivity
	Clp_getColSolution
	Clp_setColSolution
	Clp_getRowPrice
	Clp_getReducedCost
	Clp_getRowLower
	Clp_getRowUpper
	Clp_getObjCoefficients
	Clp_getColLower
	Clp_getColUpper
	Clp_getObjValue
	Clp_printModel
	Clp_getSmallElementValue
	Clp_setSmallElementValue
	ClpSolve_setSpecialOption
	ClpSolve_getSpecialOption
	ClpSolve_setSolveType
	ClpSolve_getSolveType
	ClpSolve_setPresolveType
	ClpSolve_getPresolveType
	ClpSolve_getPresolvePasses
	ClpSolve_getExtraInfo
	ClpSolve_setInfeasibleReturn
	ClpSolve_infeasibleReturn
	ClpSolve_doDual
	ClpSolve_setDoDual
	ClpSolve_doSingleton
	ClpSolve_setDoSingleton
	ClpSolve_doDoubleton
	ClpSolve_setDoDoubleton
	ClpSolve_doTripleton
	ClpSolve_setDoTripleton
	ClpSolve_doTighten
	ClpSolve_setDoTighten
	ClpSolve_doForcing
	ClpSolve_setDoForcing
	ClpSolve_doImpliedFree
	ClpSolve_setDoImpliedFree
	ClpSolve_doDupcol
	ClpSolve_setDoDupcol
	ClpSolve_doDuprow
	ClpSolve_setDoDuprow
	ClpSolve_doSingletonColumn
	ClpSolve_setDoSingletonColumn
	ClpSolve_presolveActions
	ClpSolve_setPresolveActions
	ClpSolve_substitution
	ClpSolve_setSubstitution	
	
