
/***************************************************************************/
/*           HERE DEFINE THE PROJECT SPECIFIC PUBLIC MACROS                */
/*    These are only in effect in a setting that doesn't use configure     */
/***************************************************************************/

/* Version number of project */
#define COINMP_VERSION "1.8.4"

/* Major Version number of project */
#define COINMP_VERSION_MAJOR 1

/* Minor Version number of project */
#define COINMP_VERSION_MINOR 8

/* Release Version number of project */
#define COINMP_VERSION_RELEASE 4
